@@ -1,8 +1,8 @@
 /*
  * Password quality check wrapper
  *
- * Copyright (C) 2012-2018, Red Hat, Inc. All rights reserved.
- * Copyright (C) 2012-2018, Milan Broz
+ * Copyright (C) 2012-2019 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2012-2019 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -283,12 +283,17 @@ void tools_passphrase_msg(int r)
 {
 	if (r == -EPERM)
 		log_err(_("No key available with this passphrase."));
+	else if (r == -ENOENT)
+		log_err(_("No usable keyslot is available."));
 }
 
 int tools_read_mk(const char *file, char **key, int keysize)
 {
 	int fd;
 
+	if (!keysize || !key)
+		return -EINVAL;
+
 	*key = crypt_safe_alloc(keysize);
 	if (!*key)
 		return -ENOMEM;
@@ -298,7 +303,8 @@ int tools_read_mk(const char *file, char **key, int keysize)
 		log_err(_("Cannot read keyfile %s."), file);
 		goto fail;
 	}
-	if ((read(fd, *key, keysize) != keysize)) {
+
+	if (read_buffer(fd, *key, keysize) != keysize) {
 		log_err(_("Cannot read %d bytes from keyfile %s."), keysize, file);
 		close(fd);
 		goto fail;
